import java.awt.Shape;
import java.awt.geom.AffineTransform;


/**
 * Abstrakte Klasse Moebel
 * 
 * @author Claus Albowski
 * @version 1.3  (22.01.05)
 */

public abstract class Moebel
{
  protected int xPosition;
  protected int yPosition;
  protected int orientierung;
  protected String farbe;
  protected boolean istSichtbar;
  protected int breite;
  protected int tiefe;


  /**
   * Berechnet das zu zeichnende Shape anhand der gegebenen Daten
   */
  protected abstract Shape gibAktuelleFigur();
  
  /**
   * Transformiert das Shape, so dass es die richtige Position
   * und Orientierung einnimmt.
   */
  protected Shape transformiere(Shape shape)
  {
      AffineTransform t = new AffineTransform();
      t.translate(xPosition, yPosition);
      t.rotate(Math.toRadians(orientierung),breite/2,tiefe/2);
      return  t.createTransformedShape(shape);
  }
  
  /**
   * Hole die X-Koordinate des Mittelpunktes
   * [ Hilfsfunktion f�r das Drehen. Kann bei einfachen Objekten private sein.]
   */
  protected int gibMitteX()
  {
    return xPosition+breite/2;
  }

  /**
   * Hole die Y-Koordinate des Mittelpunktes
   * [ Hilfsfunktion f�r das Drehen. Kann bei einfachen Objekten private sein.]
   */
  protected int gibMitteY()
  {
    return yPosition+tiefe/2;
  }

  /**
   * Mache dieses Objekt sichtbar. Wenn es bereits sichtbar ist, tue
   * nichts.
   */
  public void zeige()
  {
    istSichtbar = true;
    zeichne();
  }

  /**
   * Mache dieses Objekt unsichtbar. Wenn es bereits unsichtbar ist, tue
   * nichts.
   */
  public void verberge()
  {
    loesche();
    istSichtbar = false;
  }

  /**
   * Drehe auf den angegebenen Winkel
   */
  public void dreheAuf(int neuerWinkel)
  {
     loesche();
     orientierung = neuerWinkel;
     zeichne();
  }

  /**
   * Bewege dieses Objekt horizontal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeHorizontal(int entfernung)
  {
    loesche();
    xPosition += entfernung;
    zeichne();
  }

  /**
   * Bewege dieses objekt vertikal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeVertikal(int entfernung)
  {
    loesche();
    yPosition += entfernung;
    zeichne();
  }

 
  /**
   * �ndere die Farbe dieses Objektes in 'neueFarbe'.
   * G�ltige Angaben sind "rot", "gelb", "blau", "gruen",
   * "lila" und "schwarz".
   */
  public void aendereFarbe(String neueFarbe)
  {
    farbe = neueFarbe;
    zeichne();
  }

  /**
   * Zeichne dieses Objekt mit seinen aktuellen Werten auf den Bildschirm.
   */
  protected void zeichne()
  {
    if (istSichtbar)
    {
      Shape figur = gibAktuelleFigur();
      MoebelAnwendung.steuerung.zeichne ( this, farbe, figur);
    }
  }

  /**
   * L�sche dieses Objekt vom Bildschirm.
   */
  protected void loesche()
  {
    if (istSichtbar)
    {
      MoebelAnwendung.steuerung.loesche(this);
    }
  }

}
